@@ -43,7 +43,7 @@ def gth_solve(p: tarray) -> tarray:
 
         scale = np.sum(a[i, i + 1:n])
 
-        if scale <= 0.0:
+        if scale <= 0.0:  # pragma: no cover
             n = i + 1
             break
 
@@ -71,7 +71,7 @@ def rdl_decomposition(p: tarray) -> trdl:
 
     r = np.copy(vectors)
     d = np.diag(values)
-    l = npl.solve(np.transpose(r), np.eye(p.shape[0], dtype=float))
+    l = npl.solve(np.transpose(r), np.eye(p.shape[0], dtype=float))  # noqa
 
     k = np.sum(l[:, 0])
 
@@ -81,7 +81,7 @@ def rdl_decomposition(p: tarray) -> trdl:
 
     r = np.real(r)
     d = np.real(d)
-    l = np.transpose(np.real(l))
+    l = np.transpose(np.real(l))  # noqa
 
     return r, d, l
 
@@ -91,9 +91,6 @@ def slem(m: tarray) -> ofloat:
     ev = eigenvalues_sorted(m)
     indices = np.isclose(ev, 1.0)
 
-    if np.all(indices):
-        return None
-
     value = ev[~indices][-1]
 
     if np.isclose(value, 0.0):
